@@ -2,7 +2,6 @@ package XML::Parser::Expat;
 
 require 5.004;
 
-use English;
 use strict;
 use vars qw($VERSION @ISA %Handler_Setters %Encoding_Table @Encoding_Path
 	    $have_File_Spec);
@@ -13,9 +12,9 @@ use IO::Handle;
 require DynaLoader;
 
 @ISA = qw(DynaLoader);
-$VERSION = "2.26" ;
+$VERSION = "2.28" ;
 
-$have_File_Spec = do 'File/Spec.pm';
+$have_File_Spec = $INC{'File/Spec.pm'} || do 'File/Spec.pm';
 
 %Encoding_Table = ();
 if ($have_File_Spec) {
@@ -43,17 +42,19 @@ bootstrap XML::Parser::Expat $VERSION;
 		    Unparsed => \&SetUnparsedEntityDeclHandler,
 		    Notation => \&SetNotationDeclHandler,
 		    ExternEnt => \&SetExternalEntityRefHandler,
+		    ExternEntFin => \&SetExtEntFinishHandler,
 		    Entity => \&SetEntityDeclHandler,
 		    Element => \&SetElementDeclHandler,
 		    Attlist => \&SetAttListDeclHandler,
 		    Doctype => \&SetDoctypeHandler,
+		    DoctypeFin => \&SetEndDoctypeHandler,
 		    XMLDecl => \&SetXMLDeclHandler
 		    );
 
 sub new {
   my ($class, %args) = @_;
   my $self = bless \%args, $_[0];
-  $args{Used} = 0;
+  $args{_State_} = 0;
   $args{Context} = [];
   $args{Namespaces} ||= 0;
   $args{ErrorMessage} ||= '';
@@ -72,7 +73,7 @@ sub new {
 sub load_encoding {
   my ($file) = @_;
 
-  $file =~ s![^/]+$!\L$&\E!;
+  $file =~ s!([^/]+)$!\L$1\E!;
   $file .= '.enc' unless $file =~ /\.enc$/;
   unless ($file =~ m!^/!) {
     foreach (@Encoding_Path) {
@@ -130,7 +131,8 @@ sub setHandlers {
   return @ret;
 }
 
-sub xpcroak {
+sub xpcroak
+ {
   my ($self, $message) = @_;
 
   my $eclines = $self->{ErrorContext};
@@ -154,37 +156,54 @@ sub xpcarp {
 
 sub default_current {
   my $self = shift;
-  DefaultCurrent($self->{Parser});
+  if ($self->{_State_} == 1) {
+    return DefaultCurrent($self->{Parser});
+  }
 }
 
 sub recognized_string {
   my $self = shift;
-  RecognizedString($self->{Parser});
+  if ($self->{_State_} == 1) {
+    return RecognizedString($self->{Parser});
+  }
 }
 
 sub original_string {
   my $self = shift;
-  OriginalString($self->{Parser});
+  if ($self->{_State_} == 1) {
+    return OriginalString($self->{Parser});
+  }
 }
 
 sub current_line {
-  GetCurrentLineNumber($_[0]->{Parser});
+  my $self = shift;
+  if ($self->{_State_} == 1) {
+    return GetCurrentLineNumber($self->{Parser});
+  }
 }
 
 sub current_column {
-  GetCurrentColumnNumber($_[0]->{Parser});
+  my $self = shift;
+  if ($self->{_State_} == 1) {
+    return GetCurrentColumnNumber($self->{Parser});
+  }
 }
 
 sub current_byte {
-  GetCurrentByteIndex($_[0]->{Parser});
+  my $self = shift;
+  if ($self->{_State_} == 1) {
+    return GetCurrentByteIndex($self->{Parser});
+  }
 }
 
 sub base {
   my ($self, $newbase) = @_;
-  my $p = $self->{Parser};
-  my $oldbase = GetBase($p);
-  SetBase($p, $newbase) if @_ > 1;
-  $oldbase;
+  if ($self->{_State_} == 1) {
+    my $p = $self->{Parser};
+    my $oldbase = GetBase($p);
+    SetBase($p, $newbase) if @_ > 1;
+    return $oldbase;
+  }
 }
 
 sub context {
@@ -220,7 +239,9 @@ sub depth {
 sub element_index {
   my ($self) = @_;
 
-  ElementIndex($self->{Parser});
+  if ($self->{_State_} == 1) {
+    return ElementIndex($self->{Parser});
+  }
 }
 
 ################
@@ -228,13 +249,13 @@ sub element_index {
 
 sub namespace {
   my ($self, $name) = @_;
-  local($WARNING) = 0;
+  local($^W) = 0;
   $self->{Namespace_List}->[int($name)];
 }
 
 sub eq_name {
   my ($self, $nm1, $nm2) = @_;
-  local($WARNING) = 0;
+  local($^W) = 0;
 
   int($nm1) == int($nm2) and $nm1 eq $nm2;
 }
@@ -323,51 +344,46 @@ sub NamespaceEnd {
 
 ################
 
-sub is_defaulted {
-  my ($self, $attname) = @_;
-
-  warn <<'End_of_Warning;';
-The is_defaulted method no longer works.
-Please use the specified_attr method instead. The is_defaulted method will
-be removed altogether in a future version.
-End_of_Warning;
-  return 0;
-}
-
 sub specified_attr {
   my $self = shift;
   
-  GetSpecifiedAttributeCount($self->{Parser});
+  if ($self->{_State_} == 1) {
+    return GetSpecifiedAttributeCount($self->{Parser});
+  }
 }
 
 sub finish {
   my ($self) = @_;
-  my $parser = $self->{Parser};
-  UnsetAllHandlers($parser);
+  if ($self->{_State_} == 1) {
+    my $parser = $self->{Parser};
+    UnsetAllHandlers($parser);
+  }
 }
 
 sub position_in_context {
   my ($self, $lines) = @_;
-  my $parser = $self->{Parser};
-  my ($string, $linepos) = PositionContext($parser, $lines);
+  if ($self->{_State_} == 1) {
+    my $parser = $self->{Parser};
+    my ($string, $linepos) = PositionContext($parser, $lines);
 
-  return '' unless defined($string);
+    return '' unless defined($string);
 
-  my $col = GetCurrentColumnNumber($parser);
-  my $ptr = ('=' x ($col - 1)) . '^' . "\n";
-  my $ret;
-  my $dosplit = $linepos < length($string);
+    my $col = GetCurrentColumnNumber($parser);
+    my $ptr = ('=' x ($col - 1)) . '^' . "\n";
+    my $ret;
+    my $dosplit = $linepos < length($string);
   
-  $string .= "\n" unless $string =~ /\n$/;
+    $string .= "\n" unless $string =~ /\n$/;
   
-  if ($dosplit) {
-    $ret = substr($string, 0, $linepos) . $ptr
-      . substr($string, $linepos);
-  } else {
-    $ret = $string . $ptr;
+    if ($dosplit) {
+      $ret = substr($string, 0, $linepos) . $ptr
+	. substr($string, $linepos);
+    } else {
+      $ret = $string . $ptr;
+    }
+  
+    return $ret;
   }
-  
-  $ret;
 }
 
 sub xml_escape {
@@ -377,14 +393,39 @@ sub xml_escape {
   study $text;
   $text =~ s/\&/\&amp;/g;
   $text =~ s/</\&lt;/g;
-  $text =~ s/>/\&gt;/g;
   foreach (@_) {
-    my $rep = '&#' . ord($_) . ';';
-    $text =~ s/$_/$rep/g;
+    croak "xml_escape: '$_' isn't a single character" if length($_) > 1;
+
+    if ($_ eq '>') {
+      $text =~ s/>/\&gt;/g;
+    }
+    elsif ($_ eq '"') {
+      $text =~ s/\"/\&quot;/;
+    }
+    elsif ($_ eq "'") {
+      $text =~ s/\'/\&apos;/;
+    }
+    else {
+      my $rep = '&#' . sprintf('x%X', ord($_)) . ';';
+      if (/\W/) {
+	my $ptrn = "\\$_";
+	$text =~ s/$ptrn/$rep/g;
+      }
+      else {
+	$text =~ s/$_/$rep/g;
+      }
+    }
   }
   $text;
 }
 
+sub skip_until {
+  my $self = shift;
+  if ($self->{_State_} <= 1) {
+    SkipUntil($self->{Parser}, $_[0]);
+  }
+}
+
 sub release {
   my $self = shift;
   ParserRelease($self->{Parser});
@@ -398,8 +439,8 @@ sub DESTROY {
 sub parse {
   my $self = shift;
   my $arg = shift;
-  croak "Parse already in progress (Expat)" if $self->{Used};
-  $self->{Used} = 1;
+  croak "Parse already in progress (Expat)" if $self->{_State_};
+  $self->{_State_} = 1;
   my $parser = $self->{Parser};
   my $ioref;
   my $result = 0;
@@ -411,6 +452,7 @@ sub parse {
       eval {
 	$ioref = *{$arg}{IO};
       };
+      undef $@;
     }
   }
   
@@ -429,6 +471,7 @@ sub parse {
     $result = ParseString($parser, $arg);
   }
   
+  $self->{_State_} = 2;
   $result or croak $self->{ErrorMessage};
 }
 
@@ -439,7 +482,7 @@ sub parsestring {
 
 sub parsefile {
   my $self = shift;
-  croak "Parser has already been used" if $self->{Used};
+  croak "Parser has already been used" if $self->{_State_};
   local(*FILE);
   open(FILE, $_[0]) or  croak "Couldn't open $_[0]:\n$!";
   binmode(FILE);
@@ -448,6 +491,93 @@ sub parsefile {
   $ret;
 }
 
+################################################################
+package XML::Parser::ContentModel;
+use overload '""' => \&asString, 'eq' => \&thiseq;
+
+sub EMPTY  () {1}
+sub ANY    () {2}
+sub MIXED  () {3}
+sub NAME   () {4}
+sub CHOICE () {5}
+sub SEQ    () {6}
+
+
+sub isempty {
+  return $_[0]->{Type} == EMPTY;
+}
+
+sub isany {
+  return $_[0]->{Type} == ANY;
+}
+
+sub ismixed {
+  return $_[0]->{Type} == MIXED;
+}
+
+sub isname {
+  return $_[0]->{Type} == NAME;
+}
+
+sub name {
+  return $_[0]->{Tag};
+}
+
+sub ischoice {
+  return $_[0]->{Type} == CHOICE;
+}
+
+sub isseq {
+  return $_[0]->{Type} == SEQ;
+}
+
+sub quant {
+  return $_[0]->{Quant};
+}
+
+sub children {
+  my $children = $_[0]->{Children};
+  if (defined $children) {
+    return @$children;
+  }
+  return undef;
+}
+
+sub asString {
+  my ($self) = @_;
+  my $ret;
+
+  if ($self->{Type} == NAME) {
+    $ret = $self->{Tag};
+  }
+  elsif ($self->{Type} == EMPTY) {
+    return "EMPTY";
+  }
+  elsif ($self->{Type} == ANY) {
+    return "ANY";
+  }
+  elsif ($self->{Type} == MIXED) {
+    $ret = '(#PCDATA';
+    foreach (@{$self->{Children}}) {
+      $ret .= '|' . $_;
+    }
+    $ret .= ')';
+  }
+  else {
+    my $sep = $self->{Type} == CHOICE ? '|' : ',';
+    $ret = '(' . join($sep, @{$self->{Children}}) . ')';
+  }
+
+  $ret .= $self->{Quant} if $self->{Quant};
+  return $ret;
+}
+
+sub thiseq {
+  my $self = shift;
+
+  return $self->asString eq $_[0];
+}
+
 ################################################################
 package XML::Parser::ExpatNB;
 
@@ -456,15 +586,6 @@ use Carp;
 
 @ISA = qw(XML::Parser::Expat);
 
-sub new {
-  my ($class, %args) = @_;
-
-  my $self = $class->SUPER::new(%args);
-
-  delete $self->{ErrorContext};
-  $self;
-}
-
 sub parse {
   my $self = shift;
   my $class = ref($self);
@@ -483,18 +604,6 @@ sub parsefile {
   croak "parsefile method not supported in $class";
 }
 
-sub position_in_context {
-  my $self = shift;
-  my $class = ref($self);
-  croak "position_in_context method not supported in $class";
-}
-
-sub original_string {
-  my $self = shift;
-  my $class = ref($self);
-  croak "original_string method not supported in $class";
-}
-
 sub parse_more {
   my ($self, $data) = @_;
 
@@ -513,6 +622,8 @@ sub parse_done {
     croak $msg;
   }
 
+  $self->{_State_} = 2;
+
   my $result = $ret;
   my @result = ();
   my $final = $self->{FinalHandler};
@@ -645,6 +756,11 @@ Unless standalone is set to "yes" in the XML declaration, setting this to
 a true value allows the external DTD to be read, and parameter entities
 to be parsed and expanded.
 
+=item * Base
+
+The base to use for relative pathnames or URLs. This can also be done by
+using the base method.
+
 =back
 
 =item setHandlers(TYPE, HANDLER [, TYPE, HANDLER [...]])
@@ -742,14 +858,19 @@ the contents of the external entity, or return undef, which indicates the
 external entity couldn't be found and will generate a parse error.
 
 If an open filehandle is returned, it must be returned as either a glob
-(*FOO) or as a reference to a glob (e.g. an instance of IO::Handle). The
-parser will close the filehandle after using it.
+(*FOO) or as a reference to a glob (e.g. an instance of IO::Handle).
+
+=item * ExternEntFin		(Parser)
+
+This is called after an external entity has been parsed. It allows
+applications to perform cleanup on actions performed in the above
+ExternEnt handler.
 
 =item * Entity			(Parser, Name, Val, Sysid, Pubid, Ndata)
 
-This is called when an entity is declared in the internal subset. For
-internal entities, the Val parameter will contain the value and the remaining
-three parameters will be undefined. For external entities, the Val parameter
+This is called when an entity is declared. For internal entities, the Val
+parameter will contain the value and the remaining three parameters will
+be undefined. For external entities, the Val parameter
 will be undefined, the Sysid parameter will have the system id, the Pubid
 parameter will have the public id if it was provided (it will be undefined
 otherwise), the Ndata parameter will contain the notation for unparsed
@@ -761,14 +882,15 @@ set, then this handler will not be called for unparsed entities.
 
 =item * Element			(Parser, Name, Model)
 
-The element handler is called when an element declaration is found in the
-internal subset. Name is the element name, and Model is the content model
-as a string.
+The element handler is called when an element declaration is found. Name is
+the element name, and Model is the content model as an
+XML::Parser::ContentModel object. See L<"XML::Parser::ContentModel Methods">
+for methods available for this class.
 
 =item * Attlist			(Parser, Elname, Attname, Type, Default, Fixed)
 
-This handler is called for each attribute in an ATTLIST declaration found in
-the internal subset. So an ATTLIST declaration that has multiple attributes
+This handler is called for each attribute in an ATTLIST declaration.
+So an ATTLIST declaration that has multiple attributes
 will generate multiple calls to this handler. The Elname parameter is the
 name of the element with which the attribute is being associated. The Attname
 parameter is the name of the attribute. Type is the attribute type, given as
@@ -781,21 +903,22 @@ with a quote character). If Fixed is true, then this is a fixed attribute.
 This handler is called for DOCTYPE declarations. Name is the document type
 name. Sysid is the system id of the document type, if it was provided,
 otherwise it's undefined. Pubid is the public id of the document type,
-which will be undefined if no public id was given. Internal is the internal
-subset, given as a string. If there was no internal subset, it will be
-undefined. Internal will contain all whitespace, comments, processing
-instructions, and declarations seen in the internal subset. The declarations
-will be there whether or not they have been processed by another handler
-(except for unparsed entities processed by the Unparsed handler). However,
-comments and processing instructions will not appear if they've been processed
-by their respective handlers.
+which will be undefined if no public id was given. Internal will be
+true or false, indicating whether or not the doctype declaration contains
+an internal subset.
+
+=item * DoctypeFin		(Parser)
+
+This handler is called after parsing of the DOCTYPE declaration has finished,
+including any internal or external DTD declarations.
 
 =item * XMLDecl			(Parser, Version, Encoding, Standalone)
 
-This handler is called for xml declarations. Version is a string containg
+This handler is called for XML declarations. Version is a string containg
 the version. Encoding is either undefined or contains an encoding string.
-Standalone will be either true, false, or undefined if the standalone attribute
-is yes, no, or not made respectively.
+Standalone is either undefined, or true or false. Undefined indicates
+that no standalone parameter was given in the XML declaration. True or
+false indicates "yes" or "no" respectively.
 
 =back
 
@@ -844,19 +967,23 @@ currently bound, '#default' appears in the list.
 Returns the string from the document that was recognized in order to call
 the current handler. For instance, when called from a start handler, it
 will give us the the start-tag string. The string is encoded in UTF-8.
+This method doesn't return a meaningful string inside declaration handlers.
 
 =item original_string
 
 Returns the verbatim string from the document that was recognized in
 order to call the current handler. The string is in the original document
-encoding.
+encoding. This method doesn't return a meaningful string inside declaration
+handlers.
 
 =item default_current
 
 When called from a handler, causes the sequence of characters that generated
 the corresponding event to be sent to the default handler (if one is
 registered). Use of this method is deprecated in favor the recognized_string
-method, which you can use without installing a default handler.
+method, which you can use without installing a default handler. This
+method doesn't deliver a meaningful string to the default handler when
+called from inside declaration handlers.
 
 =item xpcroak(message)
 
@@ -889,12 +1016,15 @@ NEWBASE is supplied, changes the base to that value.
 
 =item context
 
-Returns a list of element names that represent open
-elements, with the last one being the innermost.
+Returns a list of element names that represent open elements, with the
+last one being the innermost. Inside start and end tag handlers, this
+will be the tag of the parent element.
 
 =item current_element
 
-Returns the name of the innermost currently opened element.
+Returns the name of the innermost currently opened element. Inside
+start or end handlers, returns the parent of the element associated
+with those tags.
 
 =item in_element(NAME)
 
@@ -917,10 +1047,18 @@ Returns the size of the context list.
 =item element_index
 
 Returns an integer that is the depth-first visit order of the current
-element. This will be zero outside of the root element. Note that
-in start and end handlers, you're outside of the element that the
-corresponding tags delimit. For example, this would return 0 when
-called from the start handler for the root element start tag.
+element. This will be zero outside of the root element. For example,
+this will return 1 when called from the start handler for the root element
+start tag.
+
+=item skip_until(INDEX)
+
+INDEX is an integer that represents an element index. When this method
+is called, all handlers are suspended until the start tag for an element
+that has an index number equal to INDEX is seen. If a start handler has
+been set, then this is the first tag that the start handler will see
+after skip_until has been called.
+
 
 =item position_in_context(LINES)
 
@@ -986,13 +1124,62 @@ XML::Parser::Expat directly, then it's your responsibility to call it.
 
 =back
 
+=head2 XML::Parser::ContentModel Methods
+
+The element declaration handlers are passed objects of this class as the
+content model of the element declaration. They also represent content
+particles, components of a content model.
+
+When referred to as a string, these objects are automagicly converted to a
+string representation of the model (or content particle).
+
+=over 4
+
+=item isempty
+
+This method returns true if the object is "EMPTY", false otherwise.
+
+=item isany
+
+This method returns true if the object is "ANY", false otherwise.
+
+=item ismixed
+
+This method returns true if the object is "(#PCDATA)" or "(#PCDATA|...)*",
+false otherwise.
+
+=item isname
+
+This method returns if the object is an element name.
+
+=item ischoice
+
+This method returns true if the object is a choice of content particles.
+
+
+=item isseq
+
+This method returns true if the object is a sequence of content particles.
+
+=item quant
+
+This method returns undef or a string representing the quantifier
+('?', '*', '+') associated with the model or particle.
+
+=item children
+
+This method returns undef or (for mixed, choice, and sequence types)
+an array of component content particles. There will always be at least
+one component for choices and sequences, but for a mixed content model
+of pure PCDATA, "(#PCDATA)", then an undef is returned.
+
+=back
+
 =head2 XML::Parser::ExpatNB Methods
 
 The class XML::Parser::ExpatNB is a subclass of XML::Parser::Expat used
-for non-blocking access to the expat library. The Expat methods
-position_in_context and original_string are disabled for this class and
-it doesn't honor the ErrorContext option. It has the following additional
-methods:
+for non-blocking access to the expat library. It does not support the parse,
+parsestring, or parsefile methods, but it does have these additional methods:
 
 =over 4
 
